﻿using AngularNote.Models;
using DotNetNote.Common;
using DotNetNote.Components;
using DotNetNote.Models;
using DotNetNote.Models.Exams;
using DotNetNote.Models.RecruitManager;
using DotNetNote.Services;
using DotNetNote.Settings;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http; // ASP.NET Core 인증
using Microsoft.AspNetCore.SpaServices.Webpack;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
//using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Logging;
using System;
//using Newtonsoft.Json.Serialization;
//using System.IO;

namespace DotNetNote
{
    public class Startup
    {
        public Startup(IHostingEnvironment env)
        {
            //[!] Configuration 
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile(
                    "appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile(
                    $"appsettings.{env.EnvironmentName}.json", optional: true)
                //[!] Configuration : Strongly Typed Configuration Setting
                //    추가 환경 설정 파일 지정
                .AddJsonFile(
                    $"Settings\\DotNetNoteSettings.json", optional: true)
                .AddEnvironmentVariables();

            Configuration = builder.Build();
        }

        public IConfigurationRoot Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            //[!] Configuration: JSON 파일의 데이터를 POCO 클래스에 주입
            services.Configure<DotNetNoteSettings>(
                Configuration.GetSection("DotNetNoteSettings"));

            //[!] 디렉터리 브라우징 기능 제공(옵션)
            services.AddDirectoryBrowser();

            //[DNN] TempData[] 개체 사용
            // In-Memory 캐싱 
            services.AddMemoryCache();

            //[!] 세션 개체 사용
            //services.AddSession(); 
            // Session 개체 사용시 옵션 부여 
            services.AddSession(options =>
            {
                // 세션 유지 시간
                options.IdleTimeout = TimeSpan.FromMinutes(30);
            });


            //[User][9] Policy 설정
            services.AddAuthorization(options =>
            {
                // Users Role이 있으면, Users Policy 부여
                options.AddPolicy(
                    "Users", policy => policy.RequireRole("Users"));
                // Users Role이 있고 UserId가 "Admin"이면 "Administrators" 부여
                options.AddPolicy(
                    "Administrators",
                        policy => policy
                            .RequireRole("Users")
                            .RequireClaim("UserId", // 대소문자 구분
                                Configuration
                                    .GetSection("DotNetNoteSettings")
                                    .GetSection("SiteAdmin").Value)
                            );
            });


            //[CORS] CORS 설정
            services.AddCors(options =>
            {
                options.AddPolicy("AllowAnyOrigin",
                    builder => builder
                    .AllowAnyOrigin()
                    .AllowAnyMethod()
                    .AllowAnyHeader());
            });


            //[MVC] MVC 서비스 추가 및 JSON 렌더링 옵션 지정
            services.AddMvc(); // ASP.NET Core의 가장 중요한 서비스
                               //.AddJsonOptions(options =>
                               //{
                               //    //[Web API] JSON 속성 첫자 소문자: ASP.NET Core 1.0 기본
                               //    options.SerializerSettings.ContractResolver =
                               //    new CamelCasePropertyNamesContractResolver();
                               //});
            #region services.AddMvc() 추가 옵션들 보관(참고용)
            ////[!] Web API에서 JSON 결과물을 대문자(모델 클래스 형태)로 변경
            //services.AddMvc().AddJsonOptions(
            //    option => option.SerializerSettings.ContractResolver =
            //        new Newtonsoft.Json.Serialization.DefaultContractResolver());

            ////[!]  JSON 첫자를 대문자로 표시(?)
            //services.AddMvc() 
            //    .AddJsonOptions(options =>
            //    {
            //        if (options.SerializerSettings.ContractResolver != null)
            //        {
            //            //[Web API] JSON 속성 첫자 대문자
            //            var castedResolver = 
            //                options.SerializerSettings.ContractResolver 
            //                    as Newtonsoft.Json.Serialization
            //                        .DefaultContractResolver;
            //            // 모델 클래스에서 대문자로 지정되었으면 그대로 출력
            //            castedResolver.NamingStrategy = null; 
            //        }
            //    });

            ////[!] 사용할 일은 없겠지만, JSON 대신에 XML을 반환하고자 한다면 아래 코드 사용
            //services.AddMvc(options =>
            //{
            //    options.OutputFormatters.Add(new XmlDataContractSerializerOutputFormatter());
            //}); 
            #endregion

            //[DI] 의존성 주입(Dependency Injection)
            DependencyInjectionContainer(services);
        }

        /// <summary>
        /// 의존성 주입 관련 코드만 따로 모아서 관리
        /// - 리포지토리 등록
        /// </summary>
        private void DependencyInjectionContainer(IServiceCollection services)
        {
            //[Demo] DemoFinder 의존성 주입: 기본 내장된 DI 컨테이너 사용
            services.AddTransient<DotNetNote.Models.DataFinder>();

            //[DI] InfoService 클래스 의존성 주입: 30.3
            services.AddSingleton<InfoService>();
            services.AddSingleton<IInfoService, InfoService>();

            //[DI(Dependency Injection)] 서비스 등록: 30.4
            //services.AddTransient<CopyrightService>();
            services.AddTransient<ICopyrightService, CopyrightService>();

            //[DI] @inject 키워드로 뷰에 직접 클래스의 속성 또는 메서드 값 출력
            services.AddSingleton<CopyrightService>();

            //[DI] GuidService 등록
            services.AddTransient<GuidService>();
            services.AddTransient<IGuidService, GuidService>();
            services.AddTransient<ITransientGuidService, TransientGuidService>();
            services.AddScoped<IScopedGuidService, ScopedGuidService>();
            services.AddSingleton<ISingletonGuidService, SingletonGuidService>();



            //[DNN][!] Configuration 개체 주입: 
            //    IConfiguration 또는 IConfigurationRoot에 Configuration 개체 전달
            //    appsettings.json 파일의 데이터베이스 연결 문자열을 
            //    리포지토리 클래스에서 사용할 수 있도록 설정
            services.AddSingleton<IConfiguration>(Configuration);


            //[User][5] 회원 관리
            services.AddTransient<IUserRepository, UserRepository>();
            // LoginFailedManager
            services.AddTransient<ILoginFailedRepository, LoginFailedRepository>();
            services.AddTransient<ILoginFailedManager, LoginFailedManager>();


            //[CommunityCamp] 모듈 서비스 등록
            services.AddTransient<ICommunityCampJoinMemberRepository,
                CommunityCampJoinMemberRepository>();


            //[DNN][1] 게시판 관련 서비스 등록            
            //[1] 생성자에 문자열로 연결 문자열 지정
            //services.AddSingleton<INoteRepository>(
            //  new NoteRepository(
            //      Configuration["Data:DefaultConnection:ConnectionString"]));            
            //[2] 의존성 주입으로 Configuration 주입
            //[a] NoteRepository에서 IConfiguration으로 데이터베이스 연결 문자열 접근
            services.AddTransient<INoteRepository, NoteRepository>();
            //[b] CommentRepository의 생성자에 데이터베이스 연결문자열 직접 전송
            services.AddSingleton<INoteCommentRepository>(
                new NoteCommentRepository(
                    Configuration["ConnectionStrings:DefaultConnection"]));

            services
                .AddTransient<MaximServiceRepository, MaximServiceRepository>();

            //[Tech] 기술 목록
            services.AddTransient<ITechRepository, TechRepository>();

            //[Attendee] 참석자 목록 - 서비스 등록
            services.AddTransient<IAttendeeRepository, AttendeeRepository>();

            //[RecruitManager] 
            services.AddTransient<
                IRecruitSettingRepository, RecruitSettingRepository>();
            services.AddTransient<IRecruitRegistrationRepository,
                RecruitRegistrationRepository>();

            //[IdeaManager] 
            services.AddTransient<IIdeaRepository, IdeaRepository>();


            //[One], [Two], ... 
            services.AddTransient<IOneRepository, OneRepository>();
            services.AddTransient<ITwoRepository, TwoRepository>();
            services.AddTransient<IThreeRepository, ThreeRepository>();
            services.AddTransient<IFourRepository, FourRepository>();
            services.AddTransient<IFiveRepository, FiveRepository>();


            // Exams: Questions, ... 
            services.AddTransient<IQuestionRepository, QuestionRepository>();

            // Heros: HeroManager 
            services.AddTransient<IHeroRepository, HeroRepository>();

            // 캐릭터 설정
            services.AddTransient<ICharacterRepository, CharacterRepository>();

            // 포인트 관리
            //services.AddTransient<IPointRepository, PointRepository>(); // DB 사용
            services.AddTransient<IPointRepository, PointRepositoryInMemory>(); // 인-메모리 사용
            services.AddTransient<IPointLogRepository, PointLogRepository>();

            // 사용자 정보 보기 전용 컴포넌트
            services.AddTransient<IUserModelRepository, UserModelRepository>();

        }

        public void Configure(
            IApplicationBuilder app, 
            IHostingEnvironment env, 
            ILoggerFactory loggerFactory)
        {
            //[!] 기본 제공 로깅
            loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            loggerFactory.AddDebug();

            //[0] 파이프라인 영역: 
            //  HTTP 파이프라인에 필요한 기능을 모듈 단위의 미들웨어로 추가해서 사용
            if (env.IsDevelopment())
            {
                //[!] 예외 발생시 좀 더 자세한 정보 표시
                app.UseDeveloperExceptionPage(); //[가]
                app.UseBrowserLink();

                // 앵귤러를 사용하기 위한 웹 팩 미들웨어 추가
                app.UseWebpackDevMiddleware(new WebpackDevMiddlewareOptions {
                    HotModuleReplacement = true
                });
            }
            else
            {
                ////[!] SSL 사용
                //app.Use(async (context, next) => {
                //    if (context.Request.IsHttps)
                //    {
                //        await next();
                //    }
                //    else
                //    {
                //        var withHttps =
                //            "https://"
                //            + context.Request.Host
                //            + context.Request.Path;
                //        context.Response.Redirect(withHttps); 
                //    }
                //});

                app.UseExceptionHandler("/Home/Error");
            }


            //[DNN] TempData 개체 사용
            app.UseSession(); //[!] 세션 개체 사용, 반드시 UseMvc() 이전에 호출되어야 함

            // 미들웨어 추가

            //// 사용자 정의 미들웨어 추가
            //app.Use(async (ctx, next) => {
            //    Console.WriteLine("Hello pipeline, {0}", ctx.Request.Path);
            //    await next();
            //});

            //[!] 샘플 페이지 출력 미들웨어 
            //app.UseWelcomePage(); // 하나의 샘플 페이지 출력

            //[!] MIME 타입 설정 : FileExtensionContentTypeProvider 클래스 사용

            //[!] 파일 서버 미들웨어 추가

            //[!] UseDefaultFiles() 미들웨어: 기본 문서 제공
            //DefaultFilesOptions options = new DefaultFilesOptions();
            //options.DefaultFileNames.Clear();
            //options.DefaultFileNames.Add("NewDefault.html");
            //app.UseDefaultFiles(options);

            ////[!] 정적 파일 폴더 추가 : Azure Web Apps에서 지원하지 않음
            //// wwwroot에 HTML/CSS/JavaScript/Images 등의 정적인 파일을 실행
            //// Microsoft.AspNetCore.StaticFiles 참조 추가
            //app.UseStaticFiles(
            //    new StaticFileOptions()
            //    {
            //        FileProvider = new PhysicalFileProvider(
            //            Path.Combine(Directory.GetCurrentDirectory()
            //                , @"MyStaticFiles")),
            //        RequestPath = new PathString("/StaticFiles")
            //    }
            //);

            //[!] UseFileServer() 미들웨어 : 정적 파일 및 디렉터리 브라우징 표시 등 
            app.UseFileServer(); // 아래 3개 미들웨어 포함
                                 //[!] 정적 파일 실행을 위한 UseStaticFiles() 미들웨어 추가
                                 //app.UseStaticFiles(); //[다]
                                 //app.UseDirectoryBrowser();
                                 //app.UseDefaultFiles(); 

            //[!] 상태 코드 표시 
            app.UseStatusCodePages(); //[나] Status Code: 404; Not Found 형태로 상태 표시가 출력됨

            //[User][1] 쿠키 인증 사용 공식 코드: 
            // Microsoft.AspNetCore.Authentication.Cookies 패키지(어셈블리) 추가
            app.UseCookieAuthentication(
                new CookieAuthenticationOptions()
                {
                    AuthenticationScheme = "Cookies",
                    // 로그인하지 않았을 때 [Authorize]에 의해서 Login으로 이동
                    LoginPath = new PathString("/User/Login"),
                    AccessDeniedPath = new PathString("/User/Forbidden"),
                    AutomaticAuthenticate = true,
                    AutomaticChallenge = true
                }
            );

            //[!] CORS
            //app.UseCors(options => options.WithOrigins(
            //    "http://dotnetnote.azurewebsites.net/api/values"));
            app.UseCors(
                options => options.AllowAnyOrigin().WithMethods("GET"));


            //[CORS] CORS 설정
            app.UseCors("AllowAnyOrigin");


            //[MVC] 
            //[1] MVC 기본 라우트 정의
            //app.UseMvcWithDefaultRoute(); //[라]

            //[2] MVC 사용자 정의 라우트 정의 
            app.UseMvc(routes =>
            {
                //[1] ASP.NET Core를 메인으로 사용할 때
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");

                routes.MapSpaFallbackRoute(
                    name: "spa-fallback",
                    defaults: new { controller = "Home", action = "Index" });

                ////[2] Angular를 메인으로 사용할 때
                //routes.MapRoute(
                //    name: "default",
                //    template: "{controller=Angular}/{action=Index}/{id?}");

                //routes.MapSpaFallbackRoute(
                //    name: "spa-fallback",
                //    defaults: new { controller = "Angular", action = "Index" });

                ////[3] Blog를 메인으로 사용할 때
                //routes.MapRoute(
                //    name: "default",
                //    template: "{controller=Cloud}/{action=Index}/{id?}");

                //routes.MapSpaFallbackRoute(
                //    name: "spa-fallback",
                //    defaults: new { controller = "Cloud", action = "Index" });
            });
        }
    }
}
